package kochModel.factory;

import kochModel.*;

import lrs.*;
//import java.awt.*;
import java.awt.geom.*;

import fp.*;

public abstract class FractalFactory implements IFactory {
  
  private Koch proto;
  
  public Koch getProto() {
    return proto;
  }
  
  public void  setProto(Koch aProto) {
    proto = aProto;
  }
  
  public Koch makeBase(Point2D.Double a, Point2D.Double b) {
    return new Koch(a, b);
  }
  
  public Koch makeInduct(Point2D.Double a, Point2D.Double b) {
    Point2D.Double delta = new Point2D.Double((double)(b.x-a.x), (double)(b.y-a.y));
    Point2D.Double a2 = new Point2D.Double((double) a.x, (double) a.y);
    final AffineXForm affineXForm = new AffineXForm(a2, delta);
    
    IKochVisitor<Koch, Object> xfmVisitor = new IKochVisitor<Koch, Object>() {
      IKochVisitor<Koch, Object> thisXfmVisitor = this;
      public Koch baseCase(Koch host, Object... nu) {
        return new Koch(affineXForm.doXForm(host.getA()), affineXForm.doXForm(host.getB()));
      }
      
      public Koch inductCase(Koch host, Object... nu) {
        LRStruct<Koch> children = host.fold( new IFoldAlgo<Koch, LRStruct<Koch>, Object>() {
          public LRStruct<Koch> apply( Koch host, LRStruct<Koch> base, Object... nu) {
            base.insertFront(host.execute(thisXfmVisitor));
            return base;
          }
        }, new LRStruct<Koch>());
        return new Koch(affineXForm.doXForm(host.getA()), affineXForm.doXForm(host.getB()), 
                        children);
      }      
    };
    
    return proto.execute(xfmVisitor);  
  }
  
}
